﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using Syncfusion.Windows.Forms.Diagram;
using System.Drawing;
using System.Collections;


public class CustomLayoutManager : DirectedTreeLayoutManager
{
    private float hOffset;
    private float vOffset, vSpacing;
    private Model modl;

    public CustomLayoutManager()
        : base()
    {
    }
    public CustomLayoutManager(Model model, float fRotationDegree, float fVerticalOffset, float fHorizontalOffset)
        : base(model, fRotationDegree, fVerticalOffset, fHorizontalOffset)
    {
        hOffset = fHorizontalOffset;
        vOffset = fVerticalOffset;
        vSpacing = fVerticalOffset;
        modl = model;
    }

    protected override void DoGraphLayout()
    {
        PriorRoutine(modl.Nodes);

    }
    private void PriorRoutine(NodeCollection nodes)
    {
        FakeRadialLayout(nodes);
    }
    public void FakeRadialLayout(NodeCollection nodes)
    {
        NodeCollection parents = new NodeCollection();
        NodeCollection childs = new NodeCollection();
        ArrayList child = new ArrayList();
        foreach (Node node in nodes)
        {
            if (!(node is ConnectorBase) && (node.EdgesEntering.Count == 0))
            {
                parents.Add(node);
            }
        }
        PositionNodes(parents, null, true);
        ArrayList parntLst = parents.Members;
        ArrangeNodes(parntLst);

    }

    private void ArrangeNodes(ArrayList parents)
    {
        if (parents != null && parents.Count != 0)
        {
            ArrayList snglePrtchld = new ArrayList();
            ArrayList mulPrntChld = new ArrayList();
            ArrayList tempPrnt = new ArrayList();

            foreach (Node node in parents)
            {
                foreach (IGraphEdge edge in node.EdgesLeaving)
                {
                    if (edge.ToNode.EdgesEntering.Count > 1)
                    {
                        if (!(mulPrntChld.Contains(edge.ToNode)) && !(parents.Contains(edge.ToNode)))
                        {
                            mulPrntChld.Add(edge.ToNode);
                        }
                    }
                    else
                    {
                        if (!(snglePrtchld.Contains(edge.ToNode)))
                        {
                            snglePrtchld.Add(edge.ToNode);
                        }
                    }
                }

            }
            PlaceNodes(snglePrtchld, mulPrntChld, parents);
            tempPrnt.Clear();
            foreach (Node node in parents)
            {
                foreach (IGraphEdge edge in node.EdgesLeaving)
                {
                    if (!tempPrnt.Contains(edge.ToNode) && !(parents.Contains(edge.ToNode)))
                        tempPrnt.Add(edge.ToNode);
                }
            }
            ArrangeNodes(tempPrnt);


        }
    }

    private void PlaceNodes(ArrayList snglePrtchld, ArrayList mulPrntChld, ArrayList parents)
    {
        float parntLcnX = 0, parntLcnY = 0;

        ArrayList samePrnt = new ArrayList();
        if (mulPrntChld != null)
        {
            foreach (Node node in mulPrntChld)
            {
                foreach (IGraphEdge edge in node.EdgesEntering)
                {
                    if (parents.Contains(edge.FromNode))
                    {
                        Node prntNode = edge.FromNode as Node;
                        parntLcnX += prntNode.PinPoint.X;
                        parntLcnY = prntNode.PinPoint.Y + vOffset;
                    }
                }
                node.PinPoint = new PointF(parntLcnX / 2, parntLcnY);
                parntLcnX = 0;
            }
        }
        if (snglePrtchld != null)
        {
            int i = 1, frst = 0;
            float offstX = 0;
            foreach (Node node in snglePrtchld)
            {
                foreach (IGraphEdge edge in node.EdgesEntering)
                {
                    float leaveCnt = edge.FromNode.EdgesLeaving.Count;
                    if (leaveCnt == 2)
                    {
                        Node prntNode = edge.FromNode as Node;
                        float prntX = prntNode.PinPoint.X / leaveCnt;
                        if (frst == 0)
                        {
                            node.PinPoint = new PointF((prntNode.PinPoint.X - 100), prntNode.PinPoint.Y + vOffset);
                            offstX = node.Size.Width;
                            frst = 1;
                        }
                        else
                            node.PinPoint = new PointF(prntNode.PinPoint.X + 100, prntNode.PinPoint.Y + vOffset);
                    }
                    else
                    {
                        Node prntNode = edge.FromNode as Node;
                        float prntX = prntNode.PinPoint.X / leaveCnt;
                        if (frst == 0)
                        {
                            node.PinPoint = new PointF((prntX), prntNode.PinPoint.Y + vOffset);
                            offstX = node.Size.Width;
                            frst = 1;
                        }
                        else
                            node.PinPoint = new PointF(offstX, prntNode.PinPoint.Y + vOffset);
                    }
                    i++;
                    offstX += (float)(node.Size.Width * 1.5);
                }
            }
        }
    }
    private void PositionNodes(NodeCollection childs, NodeCollection parents, bool m_Parent)
    {
        float x = hOffset;
        float y = 0;
        int frst = 0;
        foreach (Node node in childs)
        {
            RectangleF bounds = node.BoundingRectangle;
            if (frst == 0)
                node.PinPoint = new System.Drawing.PointF(hOffset + hOffset, vOffset);
            else
                node.PinPoint = new System.Drawing.PointF(hOffset, vOffset);

            hOffset += bounds.Width * 2;
            if (y < bounds.Height)
                y = bounds.Height;
            frst = 1;
        }
        hOffset = x;
        vOffset += (vSpacing + y);
    }

    public override bool UpdateLayout(object contextInfo)
    {
        FakeRadialLayout(this.Model.Nodes);
        return true;
    }
}
