﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Syncfusion.Windows.Forms.Diagram;
using System.Drawing;
using System.Runtime.Serialization;
using System.Collections;

public class PersonNode : Group
{
    //members
    private string strName;
    private string strRole;
    private string strAddress;
    private Bitmap image;
    private bool bExpanded = true;
    private Syncfusion.Windows.Forms.Diagram.Rectangle filledBackgroundNode;
    private TextNode nameNode;
    private TextNode contentNode;
    private BitmapNode photoNode;
    //properties
    public string Title
    {
        get { return strName; }
        set { strName = value; nameNode.ReadOnly = false; nameNode.Text = value; nameNode.ReadOnly = true; }
    }
    public string Role
    {
        get { return strRole; }
        set { strRole = value; contentNode.ReadOnly = false; contentNode.Text = value; contentNode.ReadOnly = true; }
    }
    public string Address
    {
        get { return strAddress; }
        set { strAddress = value; contentNode.ReadOnly = false; contentNode.Text += "\n" + value; contentNode.ReadOnly = true; }
    }
    public bool IsExpanded
    {
        get { return bExpanded; }
    }
    public Bitmap Photo
    {
        get { return image; }
        set
        {
            image = value;
            if (photoNode == null)
            {
                photoNode = new BitmapNode(value, new RectangleF(filledBackgroundNode.BoundingRectangle.Right, 0, 50, 75), MeasureUnits.Pixel);
                photoNode.EditStyle.AllowSelect = false;
                this.AppendChild(photoNode);
            }
            else
                photoNode.Image = value;
        }
    }
    public FillStyle BackgroundStyle
    {
        get { return filledBackgroundNode.FillStyle; }
    }
    public int Rank
    {
        get
        {
            int rank = 1;
            IGraphNode curNode = this as IGraphNode;
            while (curNode != null)
            {
                ICollection edgesEntering = curNode.EdgesEntering;
                if (edgesEntering.Count > 0)
                {
                    IEnumerator enumEdgesEntering = edgesEntering.GetEnumerator();
                    if (enumEdgesEntering != null)
                    {
                        enumEdgesEntering.MoveNext();
                        IGraphEdge incomingEdge = enumEdgesEntering.Current as IGraphEdge;
                        if (incomingEdge != null)
                        {
                            curNode = incomingEdge.FromNode;
                            rank++;
                        }
                    }
                }
                else
                {
                    curNode = null;
                }
            }

            return rank;
        }
    }

    public NodeCollection SubNodes
    {
        get { return GetSubNodes(); }
    }
    //constructors
    public PersonNode()
        : base()
    {
        //background            
        filledBackgroundNode = new Syncfusion.Windows.Forms.Diagram.Rectangle(0, 0, 125, 75);
        filledBackgroundNode.FillStyle.Color = Color.FromArgb(255, 181, 230, 183);
        filledBackgroundNode.EditStyle.AllowSelect = false;
        nameNode = new TextNode(string.Empty, new RectangleF(10, 2, 105, 20));
        nameNode.ReadOnly = true;
        nameNode.LineStyle.LineWidth = 0;
        nameNode.FontColorStyle.Color = Color.Red;
        nameNode.FontStyle.Bold = true;
        nameNode.HorizontalAlignment = StringAlignment.Center;
        nameNode.VerticalAlignment = StringAlignment.Center;
        nameNode.EditStyle.AllowSelect = false;
        contentNode = new TextNode(string.Empty, new RectangleF(10, 25, 105, 45));
        contentNode.ReadOnly = true;
        contentNode.LineStyle.LineWidth = 0;
        contentNode.HorizontalAlignment = StringAlignment.Center;
        contentNode.VerticalAlignment = StringAlignment.Center;
        contentNode.EditStyle.AllowSelect = false;
        this.AppendChild(filledBackgroundNode);
        this.AppendChild(nameNode);
        this.AppendChild(contentNode);

        this.ShadowStyle.Visible = true;
        this.EnableCentralPort = false;
        this.DrawPorts = false;
        //Connection Point
        ConnectionPoint port = new ConnectionPoint();
        port.Position = Position.BottomCenter;
        this.Ports.Add(port);
        port = new ConnectionPoint();
        port.Position = Position.TopCenter;
        this.Ports.Add(port);
        port = new ConnectionPoint();
        port.Position = Position.MiddleLeft;
        this.Ports.Add(port);
        port = new ConnectionPoint();
        port.Position = Position.MiddleRight;
        this.Ports.Add(port);
        port = new ConnectionPoint();
        port.Position = Position.Custom;
        port.OffsetX = 0;
        port.OffsetY = 20;
        this.Ports.Add(port);
        port = new ConnectionPoint();
        port.Position = Position.Custom;
        port.OffsetX = 125;
        port.OffsetY = 20;
        this.Ports.Add(port);
    }
    public PersonNode(string name, string role, string address)
        : base()
    {
        this.Title = name;
        this.Role = role;
        this.Address = address;
    }
    public PersonNode(string name, string role, string address, Bitmap photo)
        : this(name, role, address)
    {
        photoNode = new BitmapNode(photo, new RectangleF(filledBackgroundNode.BoundingRectangle.Right, 0, 50, 75), MeasureUnits.Pixel);
        photoNode.EditStyle.AllowSelect = false;
        this.AppendChild(photoNode);
        image = photo;
    }
    public PersonNode(PersonNode src)
        : base(src)
    {
        this.strAddress = src.strAddress;
        this.strName = src.strName;
        this.strRole = src.strRole;
        this.image = src.image;
    }
    public PersonNode(SerializationInfo info, StreamingContext context)
        : base(info, context)
    {
        bExpanded = info.GetBoolean("isExpanded");
        strAddress = info.GetString("address");
        strName = info.GetString("title");
        strRole = info.GetString("role");
        image = (Bitmap)info.GetValue("photo", typeof(Bitmap));
    }

    //overrides
    public override object Clone()
    {
        return new PersonNode(this);
    }
    protected override void GetObjectData(SerializationInfo info, StreamingContext context)
    {
        base.GetObjectData(info, context);
        info.AddValue("title", strName);
        info.AddValue("role", strRole);
        info.AddValue("address", strAddress);
        info.AddValue("photo", image);
        info.AddValue("isExpanded", bExpanded);
    }

    protected override void OnMouseDoubleClick(EventArgs e)
    {
        base.OnMouseDoubleClick(e);

        // Get node root.
        Syncfusion.Windows.Forms.Diagram.Model diagramModel = this.Root;

        // Begin model update.
        diagramModel.BeginUpdate();

        // If node is expanded collapse its, else expand.
        if (bExpanded == false)
        {
            this.ExpandSubNodes(true);
        }
        else
        {
            this.CollapseSubNodes(true);
        }

        diagramModel.EndUpdate();

    }

    protected override void Render(Graphics gfx)
    {
        base.Render(gfx);

        // draw helper plus/minus symbol
        DrawPlusMinus(gfx);
    }

    //helper methods
    public void ExpandSubNodes(bool clickednode)
    {
        IGraphNode thisNode = this as IGraphNode;

        if (thisNode != null)
        {
            ICollection edgesLeaving = thisNode.EdgesLeaving;
            if (edgesLeaving != null)
            {
                foreach (IGraphEdge curEdge in edgesLeaving)
                {
                    // Get from IGraphEdge link.
                    ConnectorBase outlink = curEdge as ConnectorBase;

                    if (outlink != null)
                    {
                        // Get from link children of current symbol.
                        PersonNode cursymbol = outlink.ToNode as PersonNode;
                        if (cursymbol != null)
                        {
                            // Check for explanded and explande nodes in child symbol.
                            if (cursymbol.IsExpanded)
                                cursymbol.ExpandSubNodes(false);

                            // Set current symbol as visible.
                            cursymbol.Visible = true;
                        }

                        // Set connected link as visible too.
                        outlink.Visible = true;
                    }
                }
            }
        }

        // If this symbol is node under mouse click set node state as 
        // expaned by make plus vertical line visibly.
        if (clickednode && HasChild())
        {
            // Set symbol shapes to indicate an expanded state.
            bExpanded = true;
        }
    }
    public void CollapseSubNodes(bool clickednode)
    {
        IGraphNode thisNode = this as IGraphNode;
        if (thisNode != null)
        {
            ICollection edgesLeaving = thisNode.EdgesLeaving;
            if (edgesLeaving != null)
            {
                // Convert to collection of IGraphEdges type.
                foreach (IGraphEdge curEdge in edgesLeaving)
                {
                    // Get from IGraphEdge link.
                    ConnectorBase outlink = curEdge as ConnectorBase;
                    if (outlink != null)
                    {
                        // Get from link current symbol children.
                        PersonNode cursymbol = outlink.ToNode as PersonNode;
                        if (cursymbol.EdgesEntering.Count > 1)
                        {
                            bool expandall = false;
                            foreach (ConnectorBase cn in cursymbol.EdgesEntering)
                            {
                                if (!cn.Visible)
                                    expandall = true;
                            }
                            if (!expandall)
                            {
                                outlink.Visible = false;
                                break;
                            }
                        }
                        if (cursymbol != null)
                        {
                            // Colapse nodes in child symbol.
                            cursymbol.CollapseSubNodes(false);

                            // Set current symbol as invisible.
                            cursymbol.Visible = false;
                        }
                        // Set connected link as invisible too.
                        outlink.Visible = false;
                    }
                }
            }
        }

        // If this symbol is node under mouse click set node state as 
        // expaned by make plus vertical line visibly.
        if (clickednode && HasChild())
        {
            // Set symbol shapes to indicate a collapsed state.
            bExpanded = false;
        }
    }
    private EndPoint GetTargetEndPoint(EndPoint endPoint)
    {
        EndPoint endPointToReturn = endPoint;

        IEndPointContainer container = endPoint.Container as IEndPointContainer;

        if (container != null)
        {
            endPointToReturn = (container.HeadEndPoint == endPoint) ? container.TailEndPoint : container.HeadEndPoint;
        }

        return endPoint;
    }
    private bool HasChild()
    {
        return this.EdgesLeaving.Count > 0;
    }
    private void DrawPlusMinus(Graphics gfx)
    {
        if (HasChild())
        {
            SizeF size = this.BoundingRect.Size;

            using (Pen pen = new Pen(Color.Black, 1))
            {
                gfx.DrawEllipse(pen, new RectangleF(size.Width / 2 - 5, -20, 10, 10));
                gfx.DrawLine(pen, new PointF(size.Width / 2, -10), new PointF(size.Width / 2, 0));
                if (!bExpanded)
                {
                    //vertical line
                    gfx.DrawLine(pen, new PointF(size.Width / 2, -20), new PointF(size.Width / 2, -10));
                    //horizontal line
                    gfx.DrawLine(pen, new PointF(size.Width / 2 - 5, -15), new PointF(size.Width / 2 + 5, -15));
                }

            }
        }
    }
    private NodeCollection GetSubNodes()
    {
        NodeCollection nodes = new NodeCollection();

        foreach (IGraphEdge edge in this.EdgesLeaving)
        {
            Node toNode = edge.ToNode as Node;

            if (toNode != null)
            {
                nodes.Add(toNode);
            }
        }

        return nodes;
    }
}

